#include "pch.h"
#include "Utils.h"
#include <strsafe.h>

LPWSTR g_module_name = nullptr;

void DebugPrintf(const char* lpFormat, ...)
{
    CHAR buf[1024];
    va_list va;

    va_start(va, lpFormat);

    StringCbVPrintfA(buf, sizeof(buf), lpFormat, va);

    OutputDebugStringA(buf);
}

std::wstring GetModuleFile(HMODULE module)
{
    if (g_module_name)
        return g_module_name;
    wchar_t buffer[MAX_PATH];
    if (!GetModuleFileName(module, buffer, ARRAYSIZE(buffer)))
    {
        DebugPrintf("Error getting module file name: %d\n", GetLastError());
        return L"";
    }
    return buffer;
}


std::wstring GetExecutableFile()
{
    return GetModuleFile(nullptr);
}

std::wstring GetExecutableFileName()
{
    std::wstring file = GetExecutableFile();
    if (file.empty())
        return L"";
    auto ofs = file.rfind('\\');
    if (ofs == std::wstring::npos) {
        printf("Couldn't find last path component\n");
        return file;
    }

    return &file.c_str()[ofs + 1];
}

std::wstring GetExecutableDirectory()
{
    std::wstring file = GetExecutableFile();
    if (file.empty())
        return L"";
    auto ofs = file.rfind('\\');
    if (ofs == std::wstring::npos) {
        printf("Couldn't find last path component\n");
        return L"";
    }
    file.erase(ofs);
    return file;
}